# Mesures de réseaux d'interaction

`Introduction: `

Ici nous allons analyser un réseau de collaboration scientifique en informatique pour de mesurer un certain nombre de caractéristiques qui constituent ce réseau.  On extrait le réseau de DBLP, disponible sur SNAP(Stanford Network Analysis Project). 
Pour se faire, on utilise GraphStream qui permet de mesurer de nombreuses caractéristiques d'un réseau. La plupart de ces mesures sont implantées comme des méthodes statiques dans la classe Toolkit.

_`1- Lecture des données avec GraphStream :`_

Pour commencer, nous téléchargeons le fichier contenant les données (la structure de notre réseau)  de DBLP (com-dblp.ungraph.txt.gz) , puis pour le lire nous avons instancier un FileSourceEdge() de GraphStream qui sait lire le format de graphique type arêtes (notre fichier com-dblp.ungraph.txt), et on lu notre fichier en utilisant la fonction readAll()  qui fait la lecture du fichier en entier et en une seule instruction.

_`2- Les Mesure de base du réseau :`_

Nous avons fait quelques mesures sur le graphe en utilisant les méthodes existantes de graphStream. 

_1- Nombre de noeuds :_

Pour trouver le nombre de noeuds de notre graphe nous avons utilisé la fonction getNodeCount()  qui renvoit le nombre de noeuds du graphe en question (notre graphe) .
le nombre de noeud de ce graphe est (`N`) : **317080**

_2- Nombre de liens (arêtes):_

Pour trouver le nombre de liens de notre graphe nous avons utilisé la fonction getEdgeCount()  qui renvoit le nombre de liens (arêtes) du graphe en question (notre graphe) .
le nombre de liens de ce graphe est (`L`) : **1049866**

_3- Degré moyen:_

Pour trouver le degré moyen de notre graphe nous avons utilisé la fonction averageDegree(notre graphe) de la classe Toolkit qui renvoit le degré moyen du graphe passer en paramétre (notre graphe) .
le degré moyen de ce graphe est (`<k>`) : **6.62208890914917**

_3- Coefficient de clustering:_ 

Pour trouver le coefficient de clustering qui est la moyenne du coefficient de clustering de tous les sommets qui ont un degré supérieur ou égal à 2  de notre graphe, nous avons utilisé la fonction averageClusteringCoefficient(notre graphe) de la classe Toolkit qui renvoit le degré moyen du graphe passer en paramétre .
le coefficient clustering est (`<C>` ): **0.6324308280637396**


_1-2-_  la probabilité qu'un noeud quelconque du graphe soit relié à un autre noeud dans le cas d'un graphe aléatoire G(N,p) est la même pour tous les noeuds ce qui veut dire que la valeur du coefficient de clustering vaut la meme chose  que cette probabilité => `Ci = p`.
sachant que `p = <k>/N`,  Alors le coefficient de clustering  pour un réseau aléatoire de la même taille et du même degré moyen est la moyenne du coefficient de clustering de tous les sommets qui ont un degré supérieur ou égal à 2  =  **0.000020884599814** 

_`3 - La Connexité :`_

_3-1-_ Un réseau connexe est un réseau dont tout les noeuds sont connectés entre eux et pour le vérifier sur notre graphe, Il existe une fonction de GraphStream qui renvoit true si le graphe est connexe, sinon false, en appliquant cette fonction Toolkit.isConnected(notre graphe) sur notre graphe, elle nous renvoit true alors ce graphe est connexe.

_3-2-_ On a vu en cours que : pour qu'un réseau aléatoire soit connexe il faut vérifier la notion du degré moyen du graphe qui doit être supérieur `<k>`  au ln(Nombre de noeud du graphe) `<k> > ln(N)`. Alors pour un réseau aléatoire de même taille et de même degré moyen cela donne :
`<k> = 6.62208890914917 & 
ln(N) = 12.666909387`
On voit bien que `〈k〉> lnN(p>lnN/N)`.
 Alors le réseau aléatoire de même taille et de même degré moyen n'est pas connexe car l'inégalité n'est pas vérifier.

_3-3-_ Alors pour qu'un  réseau aléatoire avec cette taille devient connexe il faut que le degré moyen de ce réseau soit supérieur à 12.666909387

_`4- La distribution des degrés :`_

La distribution des degres est la probabilité qu'un noeud choisi au hasard ait degré `k`.
Alors pour calculer la  distribution des degrés, nous avons créé une méthode disDeg() qui calcul la probabilité  qu’un noeud choisi au hasard ait degré `k`, à cet effet nous avons fait appel a la fontion
`Toolkit.degreeDistribution(notre graph)` qui renvoit un tableau où chaque indice de cellule représente le degré, et la valeur de la cellule le nombre de noeud ayant ce degré, nous avons stocké son résultat dans un tableau d'entiers degreProba[] 
et puis on a créé un fichier texte ou chaque ligne représente  l'association de la probabilité qu'un noeud choisi au hasard ait degré `k` à son degré `k` par exemple
`6 0.05796013624321938` => `0.05796013624321938` correspond à la distribution de degré `6`.

Alors à partir du fichier générer `distributionDeg.txt` contenant la distribution des degres, on trace le graphe correspondant a la probabilité qu’un noeud choisi au hasard ait degré `k` en fonction de son degré `k`.
comme on peut le voir en :

_`Echelle linéaire :`_

Pour le générer nous avons utilisé le script : `scriptDistributionDegreLine.gnuplot`

![img_1.png](distribution/img_1.png)


_`Echelle log-log :`_

Pour le générer j'ai utilisé le script : `scriptDistributionDegreLog.gnuplot`

![img_2.png](distribution/img_2.png)

En analysant le graphe on peut voir une ligne droite pendant plusieurs ordres de grandeur. Donc cela nous conduit 
à dire que la distribution de degré suit une loi de puissance.



_`la distribution de Poisson :`_

Pour le générer nous avons utilisé le script : `scriptDistributionDePoisson.gnuplot`

![img.png](distribution/img.png)

_`loi de puissance :`_

Pour le générer nous avons utilisé le script : `scriptLoiDePuissance.gnuplot`

![img_3.png](distribution/img_3.png)

d'aprés le fit (fichier générer par la commande `fit` de `gnuplot`)  on peut voir que `gamma=  2.70539  +/- 0.04437` 

_`5- La distance:`_

La distance moyenne d'un graphe est la somme des disatances d'un noeud (noeud source) vers tous les autres noeuds du grphe
Alors pour trouver la distance moyenne de notre echantillon de 1000 noeuds, nous avons créé une méthode qui fait ce calcul, elle prend 1000 noeuds du reseau au hasard
 puis on fait la somme de ses distances vers les autres noeuds du graphe .

Alors le résultat sera : **6.792355134350952**.

Donc avec une telle distance moyenne on peut dire que l'hypothése de de six degrés de séparation est confirmé.

Pour savoir s'il s'agit d'un réseau "petit monde" ?  Oui est un réseau petit monde car la distance entre deux
nœuds choisis au hasard est courte.

La distance moyenne dans un reseau aleatoire avec les memes caracteristiques :
`d = ln(N)/ln(k) = ln(317080)/ln(6.62208890914917) = 6.70061181886`

_`la distribution des distances :`_

Pour le générer nous avons utilisé le script : `scriptDistributionDistances.gnuplot`


![img_5.png](distribution/img_5.png)

 On peut voir d'aprés la courbe tracé que son sommet indique le plus grand nombre de
noeuds qui partage la meme distance avec un autre noeud. et d'aprés le graphe on peut déduire que
 la loi de distribution de distance suit une loi Binomiale.

_`6- Le générateurs de GraphStream :`_

_`a méthode d'attachement préférentiel (Barabasi-Albert) :`_

Pour générer le graphique en utilisant la avec la méthode d'attachement préférentiel (Barabasi-Albert) 
On a crée la méthode `generateurBarabasi(int nbNoeuds, int degreeMoyen)` qui prend en paramétre le nombre de noeuds du graphe ainsi le degré moyen 

Les mesures que nous avons fait sur ce graphe se présentent ainsi : 

le nombre de noeud de ce graphe est: **317082**

le nombre de liens de ce graphe est: **1267694**

le degré moyen de ce graphe est: **7.996001243591309**

le clustering coeficient est: **4.34868464589236E-4**


_`Le générateur aléatoire :`_

Pour générer le graphique en utilisant le générateur aléatoire de graphe 
nous avons créé la méthode `generateurRandom(int nbNoeuds, int degreeMoyen)` qui prend en paramétre le nombre de noeuds du graphe ainsi le degré moyen

Les mesures que nous avons fait sur ce graphe se présentent ainsi:

le nombre de noeud de ce graphe générer aléatoirement est : **317088**

le nombre de liens de ce graphe générer aléatoirement  est : **1109349**

le degré moyen de ce graphe générer aléatoirement  est  : **6.997105121612549**

le clustering coeficient de ce graphe générer aléatoirement est  : **3.211396337453094E-5**



_ Comparaison entre le reseau de Collaborateur et celui generer avec le generateur aleatoire :

le cooefficient de clustering du réseau collaborateur est élévé par rapport au reseau aléatoire.


